import os
import sys
import re
import math
import bin2gray2bin

__version__ = '1.0'

__DESCRIPTION__ = '''
vloglib is a verilog generation library.
'''

# Description:
#   generate localparam for state machine
#
# Args:
#   l_stat: list of all states
#   code: 'bin'|'dec'(default)|'hex'|'gray'|'onehot'
#
# For example:
#   st = ['S0', 'S1', 'S2', 'S3']
#   vloglib.stat_gen(st, 'bin')
#   or 
#   vloglib.stat_gen(st, 'gray')
#
def stat_gen(l_stat, code='dec'):
    #calc bit width
    stat_num = len(l_stat)
    bit_width = math.ceil(math.log2(stat_num))
 
    #get the max lenght of strings in list
    l = [len(i) for i in l_stat]
    maxlenth = max(l)
 
    #print verilog code
    for i in range(len(l_stat)):
        n = '  localparam {}'.format(l_stat[i]).ljust(13 + maxlenth)
        c = ''
        
        if code == 'bin':
            t = '{:b}'.format(i)
            t = t.rjust(bit_width, '0')
            c = ' = {}\'b{};'.format(bit_width, t)
            
        elif code == 'dec':
            c = ' = {}\'d{};'.format(bit_width, i)
            
        elif code == 'hex':
            t = '{:x}'.format(i)
            t = t.rjust(math.ceil(bit_width/4), '0')
            c = ' = {}\'h{};'.format(bit_width, t)
            
        elif code == 'gray':
            t = '{:b}'.format(i)
            t = t.rjust(bit_width, '0')
            t = bin2gray2bin.bin2gray(t, bit_width)
            c = ' = {}\'b{};'.format(bit_width, t)
            
        elif code == 'onehot':
            if i == 0:
                t = '0' * (stat_num - 1) + '1'
            else:
                t = t[1:] + '0'
            c = ' = {}\'b{};'.format(stat_num, t)
            
        print(n + c)
        
